@@ -128,7 +128,7 @@ std::vector<Window> StandaloneWindowManager::GetWindowsInStackingOrder() const
 
   return ret;
 }
-  
+
 int StandaloneWindowManager::MonitorGeometryIn(nux::Geometry const& geo) const
 {
   // TODO
@@ -168,6 +168,15 @@ bool StandaloneWindowManager::IsWindowHorizontallyMaximized(Window window_id) co
   return false;
 }
 
+bool StandaloneWindowManager::IsWindowFullscreen(Window window_id) const
+{
+  auto window = GetWindowByXid(window_id);
+  if (window)
+    return window->fullscreen;
+
+  return false;
+}
+
 bool StandaloneWindowManager::IsWindowDecorated(Window window_id) const
 {
   auto window = GetWindowByXid(window_id);
@@ -708,6 +717,7 @@ void StandaloneWindowManager::AddStandaloneWindow(StandaloneWindow::Ptr const& w
   window->visible.changed.connect([this, xid] (bool v) {v ? window_shown(xid) : window_hidden(xid);});
   window->maximized.changed.connect([this, xid] (bool v) {v ? window_maximized(xid) : window_restored(xid);});
   window->minimized.changed.connect([this, xid] (bool v) {v ? window_minimized(xid) : window_unminimized(xid);});
+  window->fullscreen.changed.connect([this, xid] (bool v) {v ? window_fullscreen(xid) : window_unfullscreen(xid);});
   window->geo.changed.connect([this, xid] (nux::Geometry const&) { window_resized(xid); window_moved(xid); });
   window->resized.connect([this, xid] { window_resized(xid); });
   window->moved.connect([this, xid] { window_moved(xid); });
@@ -732,6 +742,11 @@ std::list<StandaloneWindow::Ptr> StandaloneWindowManager::GetStandaloneWindows()
   return standalone_windows_;
 }
 
+Cursor StandaloneWindowManager::GetCachedCursor(unsigned int cursor_name) const
+{
+  return 0;
+}
+
 void StandaloneWindowManager::SetCurrentViewport(nux::Point const& vp)
 {
   current_vp_ = vp;
