@@ -38,11 +38,11 @@ Settings* settings_instance = nullptr;
 const std::string SETTINGS_NAME = "com.canonical.Unity";
 const std::string FORM_FACTOR = "form-factor";
 const std::string DOUBLE_CLICK_ACTIVATE = "double-click-activate";
-const std::string LIM_KEY = "integrated-menus";
 
 const std::string LIM_SETTINGS = "com.canonical.Unity.IntegratedMenus";
 const std::string CLICK_MOVEMENT_THRESHOLD = "click-movement-threshold";
 const std::string DOUBLE_CLICK_WAIT = "double-click-wait";
+const std::string UNFOCUSED_MENU_POPUP = "unfocused-windows-popup";
 
 const std::string UI_SETTINGS = "com.canonical.Unity.Interface";
 const std::string TEXT_SCALE_FACTOR = "text-scale-factor";
@@ -59,6 +59,9 @@ const std::string GNOME_CURSOR_SIZE = "cursor-size";
 const std::string GNOME_SCALE_FACTOR = "scaling-factor";
 const std::string GNOME_TEXT_SCALE_FACTOR = "text-scaling-factor";
 
+const std::string REMOTE_CONTENT_SETTINGS = "com.canonical.Unity.Lenses";
+const std::string REMOTE_CONTENT_KEY = "remote-content-search";
+
 const int DEFAULT_LAUNCHER_WIDTH = 64;
 const int MINIMUM_DESKTOP_HEIGHT = 800;
 const int GNOME_SETTINGS_CHANGED_WAIT_SECONDS = 1;
@@ -78,32 +81,23 @@ public:
     , ui_settings_(g_settings_new(UI_SETTINGS.c_str()))
     , ubuntu_ui_settings_(g_settings_new(UBUNTU_UI_SETTINGS.c_str()))
     , gnome_ui_settings_(g_settings_new(GNOME_UI_SETTINGS.c_str()))
+    , remote_content_settings_(g_settings_new(REMOTE_CONTENT_SETTINGS.c_str()))
     , launcher_widths_(monitors::MAX, DEFAULT_LAUNCHER_WIDTH)
     , cached_form_factor_(FormFactor::DESKTOP)
     , cursor_scale_(1.0)
     , cached_double_click_activate_(true)
     , changing_gnome_settings_(false)
     , lowGfx_(false)
+    , remote_content_enabled_(true)
   {
     parent_->form_factor.SetGetterFunction(sigc::mem_fun(this, &Impl::GetFormFactor));
     parent_->form_factor.SetSetterFunction(sigc::mem_fun(this, &Impl::SetFormFactor));
     parent_->double_click_activate.SetGetterFunction(sigc::mem_fun(this, &Impl::GetDoubleClickActivate));
+    parent_->remote_content.SetGetterFunction(sigc::mem_fun(this, &Impl::GetRemoteContentEnabled));
 
     for (unsigned i = 0; i < monitors::MAX; ++i)
       em_converters_.emplace_back(std::make_shared<EMConverter>());
 
-    // The order is important here, DPI is the last thing to be updated
-    UpdateLimSetting();
-    UpdateTextScaleFactor();
-    UpdateCursorScaleFactor();
-    UpdateFontSize();
-    UpdateDPI();
-
-    CacheFormFactor();
-    CacheDoubleClickActivate();
-
-    UScreen::GetDefault()->changed.connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &Impl::UpdateDPI))));
-
     signals_.Add<void, GSettings*, const gchar*>(usettings_, "changed::" + FORM_FACTOR, [this] (GSettings*, const gchar*) {
       CacheFormFactor();
     });
@@ -113,10 +107,6 @@ public:
       parent_->double_click_activate.changed.emit(cached_double_click_activate_);
     });
 
-    signals_.Add<void, GSettings*, const gchar*>(usettings_, "changed::" + LIM_KEY, [this] (GSettings*, const gchar*) {
-      UpdateLimSetting();
-    });
-
     signals_.Add<void, GSettings*, const gchar*>(ubuntu_ui_settings_, "changed::" + SCALE_FACTOR, [this] (GSettings*, const gchar* t) {
       UpdateDPI();
     });
@@ -155,6 +145,23 @@ public:
     signals_.Add<void, GSettings*, const gchar*>(lim_settings_, "changed", [this] (GSettings*, const gchar*) {
       UpdateLimSetting();
     });
+
+    signals_.Add<void, GSettings*, const gchar*>(remote_content_settings_, "changed::" + REMOTE_CONTENT_KEY, [this] (GSettings*, const gchar* t) {
+      UpdateRemoteContentSearch();
+    });
+
+    UScreen::GetDefault()->changed.connect(sigc::hide(sigc::hide(sigc::mem_fun(this, &Impl::UpdateDPI))));
+
+    // The order is important here, DPI is the last thing to be updated
+    UpdateLimSetting();
+    UpdateTextScaleFactor();
+    UpdateCursorScaleFactor();
+    UpdateFontSize();
+    UpdateDPI();
+
+    CacheFormFactor();
+    CacheDoubleClickActivate();
+    UpdateRemoteContentSearch();
   }
 
   void CacheFormFactor()
@@ -190,9 +197,9 @@ public:
 
   void UpdateLimSetting()
   {
-    decoration::Style::Get()->integrated_menus = g_settings_get_boolean(usettings_, LIM_KEY.c_str());
     parent_->lim_movement_thresold = g_settings_get_uint(lim_settings_, CLICK_MOVEMENT_THRESHOLD.c_str());
     parent_->lim_double_click_wait = g_settings_get_uint(lim_settings_, DOUBLE_CLICK_WAIT.c_str());
+    parent_->lim_unfocused_popup = g_settings_get_boolean(lim_settings_, UNFOCUSED_MENU_POPUP.c_str());
   }
 
   FormFactor GetFormFactor() const
@@ -320,12 +327,30 @@ public:
     }, glib::Source::Priority::LOW));
   }
 
+  void UpdateRemoteContentSearch()
+  {
+    glib::String remote_content(g_settings_get_string(remote_content_settings_, REMOTE_CONTENT_KEY.c_str()));
+    bool remote_content_enabled = ((remote_content.Str() == "all") ? true : false);
+
+    if (remote_content_enabled != remote_content_enabled_)
+    {
+      remote_content_enabled_ = remote_content_enabled;
+      parent_->remote_content.changed.emit(remote_content_enabled_);
+    }
+  }
+
+  bool GetRemoteContentEnabled() const
+  {
+    return remote_content_enabled_;
+  }
+
   Settings* parent_;
   glib::Object<GSettings> usettings_;
   glib::Object<GSettings> lim_settings_;
   glib::Object<GSettings> ui_settings_;
   glib::Object<GSettings> ubuntu_ui_settings_;
   glib::Object<GSettings> gnome_ui_settings_;
+  glib::Object<GSettings> remote_content_settings_;
   glib::Source::UniquePtr changing_gnome_settings_timeout_;
   glib::SignalManager signals_;
   std::vector<EMConverter::Ptr> em_converters_;
@@ -335,6 +360,7 @@ public:
   bool cached_double_click_activate_;
   bool changing_gnome_settings_;
   bool lowGfx_;
+  bool remote_content_enabled_;
 };
 
 //
@@ -377,7 +403,12 @@ bool Settings::GetLowGfxMode() const
 
 void Settings::SetLowGfxMode(const bool low_gfx)
 {
-  pimpl->lowGfx_ = low_gfx;
+  if (pimpl->lowGfx_ != low_gfx)
+  {
+    pimpl->lowGfx_ = low_gfx;
+
+    low_gfx_changed.emit();
+  }
 }
 
 EMConverter::Ptr const& Settings::em(int monitor) const
